package com.ruoyi.predict.mapper;

import com.ruoyi.predict.domain.Mv;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * MvMapper接口
 *
 * @author hlf
 * @date 2021-08-04
 */
public interface MvMapper {
    /**
     * 查询Mv
     *
     * @param mvNo MvID
     * @return Mv
     */
    public Mv selectMvById(Long mvNo);

    /**
     * 查询Mv列表
     *
     * @param mv Mv
     * @return Mv集合
     */
    public List<Mv> selectMvList(Mv mv);

    /**
     * 新增Mv
     *
     * @param mv Mv
     * @return 结果
     */
    public int insertMv(Mv mv);

    /**
     * 修改Mv
     *
     * @param mv Mv
     * @return 结果
     */
    public int updateMv(Mv mv);

    /**
     * 删除Mv
     *
     * @param mvNo MvID
     * @return 结果
     */
    public int deleteMvById(Long mvNo);

    int insertMvList(List<Mv> mvList);

    Mv selectMvByWturNoRef(@Param("wtgNo") Long wtgNo, @Param("measurementtypeNo") Long measurementtypeNo);

    List<Mv> selectMvListIsNot0(Mv mv);
}
